import { createCreator, tasks } from '@knapsack/creator-utils';
import { findUpPkgJson } from '@knapsack/file-utils';
import { join } from 'path';

const { pkg } = findUpPkgJson(__dirname);

const runtimeVersion = pkg.dependencies['@babel/runtime'];

export const babelCreator = createCreator({
  id: 'babel-config-starter',
  title: 'Babel Config Starter',
  description: pkg.description,
  getQuestions: async () =>
    ({
      moduleType: {
        type: 'choice',
        choices: [
          { title: 'ES Modules', value: 'es' },
          { title: 'CommonJS', value: 'cjs' },
        ],
      },
      filename: {
        type: 'choice',
        choices: [{ value: 'babel.config.json' }, { value: '.babelrc' }],
      },
      addScripts: {
        type: 'boolean',
        title: 'Update package.json with build script?',
        default: false,
      },
      addDep: {
        type: 'boolean',
        title: 'Update package.json with devDependency?',
        default: true,
      },
    } as const),
  async getTasks({ answers, config: { dest } }) {
    const { addScripts, addDep, filename, moduleType } = answers;
    const isEs = moduleType === 'es';
    const to = (path: string) => join(dest, path);
    const { pkgPath } = findUpPkgJson(dest);

    return [
      {
        title: `Creating ${filename}`,
        task: async () =>
          tasks.addFile({
            destination: to(filename),
            handlePriorFile: 'overwrite',
            contents: {
              extends: `@knapsack/babel-config-starter${isEs ? '/es' : ''}`,
            },
          }),
      },
      {
        title: 'Added script to package.json',
        task: async () => {
          if (!addScripts) return 'Did not add script to package.json';
          await tasks.updatePkgJson({
            dir: pkgPath,
            pkg: {
              scripts: {
                build:
                  'babel --extensions .js,.ts --copy-files --out-dir dist/ src/',
                start: 'npm run build -- watch',
              },
            },
          });
          return 'Did add script to package.json';
        },
      },
      {
        title: 'Added deps to package.json',
        skip: !addDep,
        task: async () => {
          if (!addDep) return 'Did not add devDependency to package.json';
          await tasks.updatePkgJson({
            dir: pkgPath,
            pkg: {
              devDependencies: {
                [pkg.name]: `^${pkg.version}`,
                '@babel/runtime': runtimeVersion || 'latest',
              },
            },
          });
          return 'Did add devDependency to package.json';
        },
      },
    ];
  },
});
